//
//  ViximoUserFilter.h
//  ViximoMobileTestApp
//
//  Created by Jonathan Klein on 1/12/12.
//  Copyright (c) 2012 None. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ViximoResource.h"


/**
 * A class used to filter users by various criteria.  Used by [Viximo sendMessage:users:callback:]
 * and [Viximo getFriends:callback:], for example, to choose the friends to interact with.
 * <p>
 * See the subclass ViximoFriendSelectorUI if you want to present the user with a pre-built
 * UI for selecting their friends.
 */
@interface ViximoUserFilter : ViximoResource {
  BOOL                    allowsSelection;
  BOOL                    gamePlayersOnly;
  BOOL                    onlineOnly;
  BOOL                    invert;
  int                     userLimit;
  NSMutableArray*         userIDs;
  NSMutableArray*         relationships;
  NSMutableArray*         groups;
}

/**
 * An array of user IDs to filter by.  Only users with these user IDs will be included.
 */
@property (nonatomic,retain) NSMutableArray *userIDs;

/**
 * An array of relationship names to filter by.  Only users who are connected to the
 * current player by one of these relationships will be included.
 * <p>
 * The relationship names specified here should be the same relationship names used in
 * calls to [Viximo setRelationship:type:bidirectional:callback:]
 */
@property (nonatomic,retain) NSMutableArray *relationships;

/**
 * An array of group names to filter by.  Only users who are friends with the current user
 * and are members of the specified groups will be included.
 * <p>
 * The group names listed here should be the same group names used in
 * calls to [Viximo addUserToGroup:callback:] and [Viximo removeUserFromGroup:callback:]
 */
@property (nonatomic,retain) NSMutableArray *groups;

/**
 * Specifies whether the filter should present the player with a user interface from which
 * to select their friends.  Used in conjunction with userLimit.
 */
@property (nonatomic,assign) BOOL allowsSelection;

/** 
 * The number of friends to limit selection to if allowsSelection is specified. If this value 
 * is set to 0, there will be no limit placed on the number of users that can be selected.
 * The default value is 1.
 */
@property (nonatomic,assign) int userLimit;

/**
 * Specifies whether the filter allows only users who are currently online.
 * The default is false.
 */
@property (nonatomic,assign) BOOL onlineOnly;

/**
 * Specifies whether the filter allows only users who have played this game before.
 * The default is false.
 */
@property (nonatomic,assign) BOOL gamePlayersOnly;

/**
 * Sets the filter to yield the inverse results, that is, friends that do not meet the criteria 
 * specified by the rest of the filter settings.  The default value is false.
 */
@property (nonatomic,assign) BOOL invert;

/**
 * Specifies a set of keys in the user's contextDictionary that should be present.
 */
@property (nonatomic,assign) NSMutableArray *contextKeys;

/**
 * Specifies a set of keys and values in a user's contextDictionary that should be present.
 */
@property (nonatomic,assign) NSMutableDictionary *contextKeysAndValues;

/**
 * Returns an autoreleased ViximoUserFilter.
 */
+ (ViximoUserFilter*)filter;

@end

/**
 * A special subclass of ViximoUserFilter which presents the user with a selection UI when invoked.
 * 
 * When used in place of a ViximoUserFilter, this class will present the user with a UI
 * that prompts them to select one or more friends.  The number of friends to select can be 
 * specified using userLimit.  The default user limit is 1.
 * <p>
 * Using other UserFilter restrictions with this class limits the friends available
 * in the friend selector UI.  So, for example, if the onlineOnly value
 * is set to true, the user will be prompted to select friend only from the set of online friends. 
 */
@interface ViximoFriendSelectorUI : ViximoUserFilter {
  NSString*               prompt;
}

/** 
 * Creates an autoreleased ViximoFriendSelectorUI which will allow for the selection of the specified number of friends.
 *
 * @param limit The number of friends to limit selection to.
 * @return The ViximoFriendsSelectorUI filter, suitable for use anywhere a ViximoUserFilter is expected.
 */
+ (ViximoFriendSelectorUI*)selectorWithLimit:(int)limit;

/**
 * A prompt to be presented to the user in the friend selector UI.
 */
@property (nonatomic,retain) NSString* prompt;

@end
