//
//  Viximo.h
//  ViximoMobileTestApp
//
//  Created by Jonathan Klein on 12/15/11.
//  Copyright (c) 2011 None. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ViximoDelegate.h"
#import "ViximoUser.h"
#import "ViximoMessage.h"
#import "ViximoActivity.h"
#import "ViximoActivityData.h"
#import "ViximoMessageData.h"
#import "ViximoInviteData.h"
#import "ViximoShareData.h"
#import "ViximoApp.h"
#import "ViximoUserFilter.h"
#import "ViximoNetworkUserInfo.h"
#import "ViximoInvite.h"


typedef enum {
  ViximoUIFriends = 0,
  ViximoUISettings,
  ViximoUIRecommendations,
  ViximoUIMessages,
  ViximoUIWelcome

} ViximoUISection;

@class ViximoViewController;

/** 
 * Viximo is the main API entry-point for the Viximo Mobile SDK.
 */
@interface Viximo : NSObject

/** @name Viximo SDK lifecycle */


/**
 * Initializes the Viximo Mobile SDK using the API key and secret that Viximo provides you with.
 * Must be called before any other method.
 * @param apiKey    The API key value provided to you by Viximo.
 * @param appSecret The application secret value provided to you by Viximo.
 * @param launchOptions   The dictionary of launch options passed in to your application delegate with application:didFinishLaunchingWithOptions:
 * @param delegate  The optional delegate object used to handle any Viximo callbacks you are interested in handling. 
 *                  This parameter may be nil if you are not interested in handling and of the action callbacks.
 */
+ (void)startWithAPIKey:(NSString*)apiKey secret:(NSString*)appSecret launchOptions:(NSDictionary*)launchOptions delegate:(id<ViximoDelegate>)delegate;

/**
 * Deprecated.  Use startWithAPIKey:secret:launchOptions:delegate: instead.
 */
+ (void)startWithAPIKey:(NSString*)apiKey secret:(NSString*)appSecret delegate:(id<ViximoDelegate>)delegate  __attribute__ ((deprecated));

/**
 * Sets the Viximo SDK to be used within the context of a particular test user.
 * This user is siloed to only the apps under your account and are unique per
 * device.  Using the same test user id on a different device will create an
 * entirely different user.
 * 
 * @param id          The unique identifier for the test user on this device.
 */
+ (void)setAsTestUser:(NSString*)userId;

/** 
 * Shuts down the Viximo SDK and releases all resources.
 * 
 * Calling this will make future invokations of the SDK require a full reinitialization through
 * startWithAPIKey:appSecret:delegate:.  As a result, calling this is not recommended during the
 * normal usage of your app.
 */
+ (void)close;

/** @name Getting information about the current user and their friends */

/** 
 * Indicates whether the user is logged in or not.
 * <p>
 * This method is typically used when you would like to access user data if available, but would not
 * like to force the user to the account creation UI if they don't have an account.
 * @return a value indicating whether or not the user has created an account and is authenticated.
 */
+ (BOOL)isLoggedIn;

/**
 * Retrieves data about the current user, invoking the provided block upon success.
 * @param complete  The block to be invoked when the user data fetch has completed.  Invoked with 
 *                  the fetched ViximoUser, or nil if the user cancels out of the account creation 
 *                  flow or an error occurs.
 */
+ (void)getUser:(void (^)(ViximoUser*))complete;

/**
 * Retrieves the current user's Viximo user ID without invoking any user interface.  This method is useful
 * for getting an identifier that can be used for sending this user messages and broadcasts, even if the user
 * has not fully completed the user login flow.
 * 
 * @param complete  The block to be invoked when the user data fetch has completed.  Invoked with 
 *                  the fetched Viximo user ID string.  Unlike getUser:, this call never displays
 *                  UI and will return a user ID string even for a user who has not gone through the
 *                  initial user onboarding process.
 */
+ (void)getUserID:(void (^)(NSString*))complete;

/** 
 * Retrieves the authentication data for a user on a given network, presenting the user
 * with a prompt to login through the network if they have not done so previously.  Addtionally, 
 * subsequent invocations of this method will always present the user with a login UI to force the refresh 
 * of the OAuth token in the event that the token has expired.  Because of this, you should not invoke 
 * this method more than one time per launch unless you explicitly want to force a refresh of the 
 * credentials and show a UI.
 * <p>
 * <b>This method only returns authentication data if the 
 * developer has provided their own network application credentials.</b>  Contact Viximo support for 
 * more information on using your own custom network application credentials.
 * <p>
 * This method can be used to ensure that users are authenticated against a desired network.  See 
 * the documentation on the Viximo web site for more information on how to incorporate this 
 * functionality into your own user flow.
 *
 * @param network     The name of the network to authenicate against.  Must be one of the constants ViximoNetworkFacebook or ViximoNetworkTwitter.
 * @param callback    The block to be invoked when the ViximoNetworkUserInfo is available.  Invoked with nil if the 
 *                    user has not logged in.  If you do not have a custom network app configured, the callback 
 *                    will be invoked with a non-nil ViximoNetworkUserInfo, but the credentials it holds will be nil.
 */
+ (void)getNetworkUserInfo:(ViximoNetwork)network callback:(void (^)(ViximoNetworkUserInfo*))complete;

/**
 * Specifies a dictionary of custom data to associate with the user.
 * <p>
 * This optional data dictionary is used to convey app specific context that can be retrieved 
 * from the current user or their friends.  This is useful for storing custom IDs or app specific
 * data associated with users such as nickname, level, etc.  
 * <p>
 * Any data set for a user can be retrieved with [ViximoUser contextDictionary].
 * <p>
 * In order to ensure efficient delivery of this custom data in messages, the dictionary 
 * is subject to the following limitations:
 *
 *   - there may be at most 5 keys
 *   - all keys and values must be NSString objects
 *   - keys may be at most 64 characters
 *   - values may be at most 256 characters
 * <p>
 * Setting the context dictionary fully overwrites any previously set context data.
 */
+ (void)saveUserContextDictionary:(NSDictionary*)inDictionary callback:(void (^)(ViximoUser*))complete;

/**
 * Retrieves data about the current user's friends, invoking the provided block block upon success.
 * @param filter    The ViximoUserFilter used to specify which friends will be fetched.  You can 
 *                  configure this filter for specific users, relationships or other values, or you can
 *                  prompt the user to select a friend themselves by using a ViximoFriendSelectorUI.
 * @param complete  The block to be invoked when the friends data fetch has completed.  Invoked with 
 *                  an array of ViximoUser friends, or nil if an error occurs.
 */
+ (void)getFriends:(ViximoUserFilter*)filter callback:(void (^)(NSArray*))complete;

/** 
 * Presents the user with the specified section of the Viximo SDK user interface.  
 * <p>
 * Can be used by the developer to prompt the user to interact with the Viximo SDK in some way, 
 * including editing user details, connecting with more friends, or viewing the message feed.
 * 
 * The ViximoUISection constants include:
 *
 *   - ViximoUISettings - Refers to the user settings UI component
 *   - ViximoUIFriends - Refers to the add friends UI component
 *   - ViximoUIRecommendations - Refers to the game recommendations UI component
 *   - ViximoUIMessages - Refers to the message center UI component
 *   - ViximoUIWelcome - Forces user through the initial user creation flow without reinitializing the user.  Useful for testing and development, but should not typically be used in production.
 *
 * @param section   The ViximoUISection of the Viximo SDK user interface to show.
 */
+ (void)showInterface:(ViximoUISection)section;

/** @name Invites, notifications messages and activity feed items */

/**
 * Deprecated.  Use inviteWithData:callback: instead.
 */
+ (void)invite:(NSString*)description callback:(void (^)(ViximoInvite*))complete  __attribute__ ((deprecated));

/**
 * Deprecated.  Use inviteWithData:callback: instead.
 */
+ (void)invite:(NSString*)description withData:(ViximoInviteData*)inviteData callback:(void (^)(ViximoInvite*))complete  __attribute__ ((deprecated));

/**
 * Displays the UI for sending app invites to the user's friends.
 *
 * @param complete      The block to be invoked upon successful completion.  The callback is passed the newly created
 *                      ViximoInvite object.
 */
+ (void)invite:(void (^)(ViximoInvite*))complete;

/**
 * Displays the UI for sending app invites to the user's friends.
 *
 * @param inviteData    The ViximoInviteData object defining the content of the invite.
 * @param complete      The block to be invoked upon successful completion.  The callback is passed the newly created
 *                      ViximoInvite object.
 */
+ (void)inviteWithData:(ViximoInviteData*)inviteData callback:(void (^)(ViximoInvite*))complete;

/**
 * Creates a new game message (notification) to be sent to a user.
 *
 * @param filter        The ViximoUserFilter source of the friends.  Use ViximoFriendSelectorUI if you 
 *                      want the user to select friends from the interface as part of this call.
 * @param messageData   The ViximoMessageData defining the content of the message.
 * @param complete      The block to be invoked upon successful completion.  The callback is passed the newly created
 *                      ViximoMessage object.  The ID of this messages can be saved and referenced later for other message 
 *                      handling methods (Such as markMessageRead:callback: and deleteMessage:callback:)
 */
+ (void)sendMessage:(ViximoMessageData*)messageData users:(ViximoUserFilter*)filter callback:(void (^)(ViximoMessage*))complete;

/**
 * Deletes a message that has been sent or scheduled for delivery.
 * 
 * @param messageID   The Viximo ID of the message to delete.
 * @param complete    The block to be invoked upon successful deletion.  Invoked with the updated ViximoMessage, or 
 *                    nil if an error occurs while updating the message.
 */
+ (void)deleteMessage:(NSString*)messageID callback:(void (^)(ViximoMessage*))complete;

/**
 * Marks that a message has been read or otherwise acted upon.
 * 
 * @param messageID   The Viximo ID of the message to mark as read.
 * @param complete    The block to be invoked upon successful deletion.  Invoked with the updated ViximoMessage, or 
 *                    nil if an error occurs while updating the message.
 */
+ (void)markMessageRead:(NSString*)messageID callback:(void (^)(ViximoMessage*))complete;


/**
 * Posts a message to the user's game activity feed.
 * 
 * @param activityData    The ViximoActivityData defining the content of the activity.
 * @param complete        The callback to be invoked with the newly created ViximoActivity.  Invoked with nil if
 *                        an error occurs.
 */
+ (void)postActivity:(ViximoActivityData*)activityData callback:(void (^)(ViximoActivity*))complete;

/**
 * Retrieves a list of recent game feed messages for the friends of a user.
 * <p>
 * The recent game feed messages can be used to implement a custom in-game activity feed
 * display.
 * 
 * @param complete        The block to be invoked with the array of ViximoActivity objects.  
 *                        Invoked with nil if an error occurs.
 */
+ (void)getRecentActivity:(void (^)(NSArray*))complete;

/**
 * Retrieves a list of recent game feed messages for the specified user's friend.
 * <p>
 * The recent game feed messages can be used to implement a custom in-game activity feed
 * display.
 * 
 * @param userId          The user id for the friend to filter feed messages from.  If nil, game
 *                        feed messages will be returned for all friends.
 * @param complete        The block to be invoked with the array of ViximoActivity objects.  
 *                        Invoked with nil if an error occurs.
 */
+ (void)getRecentActivityForUser:(NSString*)userId callback:(void (^)(NSArray*))complete;

/**
 * Fetches the unread messages for this user.
 *
 * @param complete    The block to be invoked upon successful completion.  Invoked with an array of unread ViximoMessage objects, or 
 *                    nil if an error occurs while fetching the messages.
 */
+ (void)getUnreadMessages:(void (^)(NSArray*))complete;

/**
 * Prompts the user to share content to a 3rd-party network (Facebook or Twitter).
 * <p>
 * The user does not need to be connected to the network in order to share
 * content to it.  If they're not connected, they'll be prompted to login first
 * and then be displayed a UI for sharing the content specified in this call.
 * <p>
 * If you do not specify a network to target, the user will either be automatically
 * prompted to share to the network they're connected to or select for a set of
 * networks.
 *
 * @param shareData     The ViximoShareData defining the content being shared.
 * @param callback      The callback to be invoked upon successful completion.  The callback is passed the network
 *                      that the user shared to.
 */
+ (void)share:(ViximoShareData*)shareData callback:(void (^)(ViximoNetwork))complete;


/** @name User relationships */

/** 
 * Sets a custom relationship between the current user and another user, identified by their
 * Viximo user ID.  The target player must be using the app in order for this to succeed.
 *
 * @param relationshipType  An arbitrary, developer-defined type-name used to identify the type of relationship, used
 *                          with unsetRelationship:type:callback: and ViximoUserFilter.  Must be an alphanumeric string 
 *                          (no spaces or punctuation).
 * @param userID            The Viximo user ID to associate the current user with in the new relationship.  This player
 *                          must also be using the current app in order for the relationship to be set.
 * @param bidirectional     Indicates whether the connection should be made bidirectionally or not
 * @param complete          A callback invoked with the updated ViximoUser record.  The callback is invoked with
 *                          nil if the relationship cannot be set due to an error.
 */
+ (void)setRelationship:(NSString*)userID type:(NSString*)relationshipType bidirectional:(BOOL)bidirectional callback:(void (^)(ViximoUser*))complete;

/** 
 * Removes a relationship between the current user and another user, identified by their Viximo
 * user ID.  
 *
 * @param relationshipType  The named type of the relationship to remove. 
 * @param userID            The Viximo user ID to dissociate the current user from for the given relationship type.   
 * @param complete          A callback invoked with the updated ViximoUser record.  The callback is invoked with
 *                          nil if the relationship cannot be removed due to an error.
 */
+ (void)unsetRelationship:(NSString*)userID type:(NSString*)relationshipType callback:(void (^)(ViximoUser*))complete;

/** 
 * Attempts to add a new friend to the current user's social graph.  Users must approve the creation of a 
 * friendship, so this method will display a confirmation UI to the user before making the connection.
 *
 * @param userID     The Viximo user ID to establish the friendship with.
 * @param complete   A callback to be invoked with the updated ViximoUser record.  The value is nil if the friendship
 *                   is declined or an error occurs.
 */
+ (void)addFriend:(NSString*)userID callback:(void (^)(ViximoUser*))complete;

/**
 * A convenience method for creating a new friend relationship with a user, and then a custom game relationship.
 * 
 * Functionally equivalent to addFriend:callback: followed by setRelationship:type:bidirectional:callback:. Users must approve the 
 * creation of a friendship, so this method wwill display a confirmation UI to the user before making the connection.
 *
 * @param userID             The Viximo user ID to establish the relationship with.
 * @param relationshipType   An arbitrary, developer-defined type-name used to identify the type of relationship, used
 *                           with unsetRelationship:type:callback: and ViximoUserFilter.  Must be an alphanumeric string
 *                           (no spaces or punctuation).
 * @param bidirectional      Indicates whether the connection should be made bidirectionally or not
 * @param complete           A callback to be invoked with the updated ViximoUser record.  The value is nil if the friendship
 *                           is declined or an error occurs.
 */
+ (void)addFriend:(NSString*)userID withRelationship:(NSString*)relationshipType bidirectional:(BOOL)bidirectional callback:(void (^)(ViximoUser*))complete;

/** @name Methods proxied to the Viximo Mobile SDK by your app  */

/**
 * Called by your application delegate when a push notification is received.
 * @param notification    The notification dictionary passed in to your application delegate.
 */
+ (void)didReceiveRemoteNotification:(NSDictionary*)notification;

/** 
 * Called by your application delegate when a push token is received.
 * @param token           The token data passed in to your application delegate.
 */
+ (void)didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)token;

/** 
 * Called by your application delegate when url is invoked.
 * @param url The URL invoked and passed to your application.
 */
+ (BOOL)handleOpenURL:(NSURL*)url;

/**
 * Tracks a monetization event for a user.  This data is used by the Viximo SDK to 
 * help produce analytics and targeted recommendations.
 *
 * @param amount        The dollar amount of the event.
 * @param description   A description of the event, such as the item that the user purchased.
 */
+ (void)trackPurchaseEvent:(float)amount description:(NSString*)description;

/**
 * Adds the current user to an application wide group.  If the user is already a member of the group, this will be a no-op.
 * 
 * @param groupName     The application wide group name
 * @param complete      A callback to be invoked with the updated ViximoUser record of the current user.  The value will be nil 
 *                      if an error occurs.
 */
+ (void)addUserToGroup:(NSString*)groupName callback:(void (^)(ViximoUser*))complete;

/**
 * Removes the current user from an application wide group.  If the user is not in the group, this will still execute cleanly.
 * 
 * @param groupName     The application wide group name
 * @param complete      A callback to be invoked with the updated ViximoUser record of the current user.  The value will be nil 
 *                      if an error occurs.
 */
+ (void)removeUserFromGroup:(NSString*)groupName callback:(void (^)(ViximoUser*))complete;

@end

